#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

// Defincie IO pre sledovanie iary
#define LT_R !(PIND & (1 << PD2)) // Prav senzor
#define LT_M !(PIND & (1 << PD4)) // Stredn senzor
#define LT_L !(PINB & (1 << PB2)) // av senzor

#define ENA PD5
#define ENB PD6
#define IN1 PD7
#define IN2 PB0
#define IN3 PB1
#define IN4 PB3

#define CAR_SPEED 100 // Rchlos auta pri jazde rovno
#define ROTATION_SPEED 180 // Rchlos auta pri otan
#define STOP_DURATION 1000 // Dka zastavenia
#define LINE_DETECTED_THRESHOLD 700

volatile unsigned long millis_count = 0;
volatile unsigned long lastLineTime = 0; // Premenn pre uchovanie asu poslednej detekcie iary

// Prototypy funkci
void forward();
void Mleft();
void Mright();
void stop();
void setup();
void checkLineTracking();

ISR(TIMER0_COMPA_vect) {
    millis_count++;
}

void forward() {
    OCR0A = CAR_SPEED;
    OCR0B = CAR_SPEED;
    PORTD |= (1 << IN1);  // Pohyb avmi kolesami dopredu
    PORTB |= (1 << IN4);  // Pohyb pravmi kolesami dopredu
    PORTB &= ~(1 << IN2);
    PORTB &= ~(1 << IN3);
}

void Mleft() {
    OCR0A = ROTATION_SPEED;
    OCR0B = ROTATION_SPEED;
    PORTB |= (1 << IN2);  // Pohyb avmi kolesami dozadu
    PORTB |= (1 << IN4);  // Pohyb pravmi kolesami dopredu
    PORTD &= ~(1 << IN1);
    PORTB &= ~(1 << IN3);
}

void Mright() {
    OCR0A = ROTATION_SPEED;
    OCR0B = ROTATION_SPEED;
    PORTD |= (1 << IN1);  // Pohyb avmi kolesami dopredu
    PORTB |= (1 << IN3);  // Pohyb pravmi kolesami dozadu
    PORTB &= ~(1 << IN2);
    PORTB &= ~(1 << IN4);
}

void stop() {
    OCR0A = 0;
    OCR0B = 0;
}

void setup() {
    // Nastavenie smeru pinov
    DDRD &= ~((1 << PD2) | (1 << PD4)); // Nastavenie PD2, PD4 ako vstupy
    DDRB &= ~(1 << PB2); // Nastavenie PB2 ako vstup
    DDRD |= ((1 << ENA) | (1 << ENB) | (1 << IN1)); // Nastavenie PD5-PD7 ako vstupy
    DDRB |= ((1 << IN2) | (1 << IN3) | (1 << IN4)); // Nastavenie PB0-PB1 ako vstupy

    // Inicializcia asovaa pre funkciu millis()
    TCCR0A |= (1 << WGM01);    // CTC reim
    OCR0A = 249;                // 1ms pri 16MHz
    TIMSK0 |= (1 << OCIE0A);    // Povolenie prchodu interrupcie pri zhode
    sei();                      // Povolenie globlnych interrupci
    TCCR0B |= (1 << CS01) | (1 << CS00); // Spustenie asovaa s predosledovaom 64

    // PWM nastavenie pre piny ENA a ENB (OC0A a OC0B)
    TCCR0A |= (1 << COM0A1) | (1 << COM0B1); // Nehnetci reim pre OC0A a OC0B
    TCCR0A |= (1 << WGM00) | (1 << WGM01);   // Reim rchleho PWM
    TCCR0B |= (1 << CS01);                   // Nastavenie predosledovaa na 8 pre PWM frekvenciu pribline 7,8 kHz
}

void checkLineTracking() {
    if (LT_M) {
        forward();
        millis_count = 0;
    } else if (LT_R) {
        Mleft();
        while (LT_R);
    } else if (LT_L) {
        Mright();
        while (LT_L);
    } else if(!LT_L && !LT_M && !LT_R) { 
        // Ak nie je detekovan iara viac ako 700ms
        if(millis_count - lastLineTime > 700) {
            stop(); // Zastavenie
            _delay_ms(1000); // akanie na zmenu smeru
            if(LT_L || LT_M || LT_R) return; // Ak sa objav iara, ukoni tento manver
            Mleft(); // Ak nie, pokrauj s manvrom Mleft
            _delay_ms(120); // akanie na dokonenie manvru Mleft
            if(LT_L || LT_M || LT_R) return;
            Mleft(); 
            _delay_ms(100); 
            if(LT_L || LT_M || LT_R) return;
            Mleft(); 
            _delay_ms(100);
            if(LT_L || LT_M || LT_R) return;
            Mright();
            _delay_ms(400);
            stop();
            _delay_ms(1000); 
            if(LT_L || LT_M || LT_R) return;
            Mright(); 
            _delay_ms(100); 
            if(LT_L || LT_M || LT_R) return; 
            Mright(); 
            _delay_ms(100); 
        }
    } else {
        forward();
        millis_count = 0;
    }
}

int main() {
    setup();

    for (;;) {
        checkLineTracking();
    }

    return 0;
}
